% Space carving exercise v.0.1
% CS231A, Stanford University 

function [] = voxel_coloring(dataset)
  if nargin < 1
      dataset = 'dev';
  end
  params = java.util.Hashtable;  % you may have to change this for Octave

  switch dataset
    case 'dev' % fast parameter settings for development 
      params.put('camera_matrices_filename', 'camera_matrices_dev') ; 
      params.put('camera_indices', 1:25) ;
      params.put('images_filename', 'images_dev') ; 

      grid_spacing = 0.1; 
      params.put('grid_spacing', grid_spacing) ; 
      params.put('scale_x', [0]) ; 
      params.put('scale_y', [-5:grid_spacing:5]) ; 
      params.put('scale_z', [-8:-grid_spacing:-15]);  

      params.put('pc_threshold', 1) ; 
      params.put('pc_sigma_0', 3.1); 

      % half-circle 
      ang=0:pi/100:pi;
      xg=zeros(1,length(ang));
      yg=5*cos(ang);
      zg=5*sin(ang) - 15;

      g_colors = (1:length(ang))./(length(ang))*0.75 + 0.25;
      g_colors(1) = 0;
      params.put('ground_truth', [xg', yg', zg', g_colors']) ;

    case 'dev-two-color' % two-color parameter settings 
      params.put('camera_matrices_filename', 'camera_matrices_dev_two_color') ; 
      params.put('camera_indices', 1:25) ;
      params.put('images_filename', 'images_dev_two_color') ; 

      grid_spacing = 0.05; 
      params.put('grid_spacing', grid_spacing) ; 
      params.put('scale_x', [0]) ; 
      params.put('scale_y', [-5:grid_spacing:5]) ; 
      params.put('scale_z', [-8:-grid_spacing:-15]);  

      params.put('pc_threshold', 0.0625) ; 
      params.put('pc_sigma_0', 3.1); 

      % half-circle 
      ang=0:pi/100:pi;
      xg=zeros(1,length(ang));
      yg=5*cos(ang);
      zg=5*sin(ang) - 15;

      g_colors = [ones(1,ceil(length(ang)/2),1)*0.5, ones(1,floor(length(ang)/2))*0.75];  
      params.put('ground_truth', [xg', yg', zg', g_colors']) ;

   case 'dev-highdef' % parameter settings for the dev image for the final plot 
      params.put('camera_matrices_filename', 'camera_matrices_dev') ; 
      params.put('camera_indices', 1:25) ;
      params.put('images_filename', 'images_dev') ; 

      grid_spacing = 0.05; 
      params.put('grid_spacing', grid_spacing) ; 
      params.put('scale_x', [0]) ;
      params.put('scale_y', [-5:grid_spacing:5]) ; 
      params.put('scale_z', [-8:-grid_spacing:-15]);  

      params.put('pc_threshold', 0.0625) ; 
      params.put('pc_sigma_0', 3.1); 

      % half-circle 
      ang=0:pi/100:pi;
      xg=zeros(1,length(ang));
      yg=5*cos(ang);
      zg=5*sin(ang) - 15;

      g_colors = (1:length(ang))./(length(ang))*0.75 + 0.25;
      g_colors(1) = 0;
      params.put('ground_truth', [xg', yg', zg',g_colors']) ;

   case 'test' % parameter settings for the test image for the final plot 
      params.put('camera_matrices_filename', 'camera_matrices_test') ;
      params.put('camera_indices', 1:25) ; 
      params.put('images_filename', 'images_test') ; 

      grid_spacing = 0.05; 
      params.put('grid_spacing', grid_spacing) ; 
      params.put('scale_x', [0]) ;
      params.put('scale_y', [-6:grid_spacing:6]) ; 
      params.put('scale_z', [-8:-grid_spacing:-15]);  

      params.put('pc_threshold', 0.0625) ; 
      params.put('pc_sigma_0', 3.1) ; 

   otherwise 
     throw('dataset not found. Specify dev or test or add your own dataset') 
   end 
  [camera_matrices, images] = load_dataset(params.get('camera_matrices_filename'),params.get('images_filename'), params.get('camera_indices')); 
  run(camera_matrices, images, params);

% load the camera matrices and images from saved Matlab vars and select a subset of them to use 
% you should not have to modify this 
function [camera_matrices, images] = load_dataset(camera_matrices_filename, images_filename, camera_indices) 
  num_cameras = length(camera_indices) ;

  data = load(camera_matrices_filename); 
  camera_matrices = cell(num_cameras,1) ; 
  for i=1:num_cameras 
    camera_matrices{i} = data.camera_matrices{camera_indices(i)} ; 
  end 

  data = load(images_filename); 
  images = cell(num_cameras,1) ; 
  for i=1:num_cameras 
    images{i} = data.images{camera_indices(i)} ;
  end 
  assert(length(camera_matrices) == length(images)); 

% run the actual voxel carving
% this should take < 30s for the dev params and < 3 minutes for the test params
% the primary determinant of runtime is the grid_spacing O(n^2)

function [voxels] = run(camera_matrices, images, params) 

  num_cameras = length(camera_matrices); 

  % set up the binary occlusion arrays
  occlusions = cell(num_cameras, 1) ; 
  for i=1:num_cameras 
    occlusions{i} = zeros(length(images{i}),1); 
  end

  scale_x = params.get('scale_x') ; 
  scale_y = params.get('scale_y') ; 
  scale_z = params.get('scale_z') ; 
  ii = 0; % global iterator 

  % binary grid of kept voxels to use for plotting 
  kept_voxels = zeros(length(scale_x), length(scale_y), length(scale_z));  

  for k=1:length(scale_z) 
    for j=1:length(scale_y) 
      for i=1:length(scale_x) 
        voxel_corner = [scale_x(i), scale_y(j), scale_z(k)]; 
        ii = ii + 1;

        voxel_projs = voxel_projections(voxel_corner, camera_matrices, images, params.get('grid_spacing')); 
        if photoconsistent(voxel_projs, images, occlusions, params.get('pc_threshold'), params.get('pc_sigma_0')) 
          [occlusions] = keep(voxel_projs, occlusions);
          kept_voxels(i,j,k) = 1; 
        end 

        % show progress
        if mod(ii,1000) == 0  
          sprintf('Iteration %d ', ii) 
          voxel_corner
        end 
      end 
    end 

    if nonzeros(kept_voxels) > 0 
      plot_voxels(kept_voxels, scale_x, scale_y, scale_z, params.get('ground_truth'))
    end 
  end

% in
% voxel_projections - a num_cameras cell array of projected pixel locations (Nx1, where N varies for each image) 
% occlusions - a num_cameras cell array of 400x1 occlusion masks 
% out 
% occlusions - the same occlusions array as the input but with all projections of the voxel set to 1 
function [occlusions] = keep(voxel_projections, occlusions) 
 % YOUR CODE HERE 
 % END YOUR CODE

  % this is not currently called, however, it should be useful to visualize an occlusions map layered on top of an image
function [] = plot_occlusions(images, occlusions)  
  ims = cell(length(images),1); 
  for i=1:length(camera_matrices) 
    ims{i}=bsxfun(@times,uint8(images{i}),uint8(occlusions{i})); 
  end  
  for i=1:length(ims) 
    subplot(ceil(length(camera_matrices)/3),3,i) ; 
    imshow(ims{i});
  end 

  % plot voxels along with the ground truth, if provided
function [] = plot_voxels(kept_voxels, scale_x, scale_y, scale_z, ground_truth) 

  PATCH_3Darray(kept_voxels,scale_x,scale_y,scale_z); % you may have to change this for Octave
  hold on;
  if (length(ground_truth) > 0)
    scatter3(ground_truth(:,1), ground_truth(:,2), ground_truth(:,3), 100, ground_truth(:,4));
  end

  set(gca,'DataAspectRatioMode','manual'); 
  ylim([scale_y(1),scale_y(end)]) ; 
  zlim([scale_z(end), scale_z(1)]); 
  colormap('cool'); 
  view(90, 0); 

  hold off
  pause(0.01)

